@@ -30,13 +30,9 @@
 #
 
 # Because spawn does not yet work on Windows, we fall back on the older open3 there.
-real_open3 = true
-if respond_to?(:org) && org.jruby.platform.Platform::IS_WINDOWS
-  require 'jruby/open3_windows'
-  real_open3 = false
-end
+require 'jruby/open3_windows' if JRuby::Util::ON_WINDOWS
 
-real_open3 && module Open3
+!defined?(Open3.popen3) && module Open3
 
   # Open stdin, stdout, and stderr streams and start external executable.
   # In addition, a thread to wait for the started process is created.
