@@ -33,11 +33,12 @@ class Integer
   # Returns true if +self+ is a prime number, else returns false.
   def prime?
     return self >= 2 if self <= 3
-    return false if self % 2 == 0 or self % 3 == 0
-    (5..(self**0.5).floor).step(6).each do |i|
-      if self % i == 0 || self % (i + 2) == 0
-        return false
-      end
+    return true if self == 5
+    return false unless 30.gcd(self) == 1
+    (7..Integer.sqrt(self)).step(30) do |p|
+      return false if
+        self%(p)    == 0 || self%(p+4)  == 0 || self%(p+6)  == 0 || self%(p+10) == 0 ||
+        self%(p+12) == 0 || self%(p+16) == 0 || self%(p+22) == 0 || self%(p+24) == 0
     end
     true
   end
@@ -323,7 +324,7 @@ class Prime
 
   # An implementation of +PseudoPrimeGenerator+ which uses
   # a prime table generated by trial division.
-  class TrialDivisionGenerator<PseudoPrimeGenerator
+  class TrialDivisionGenerator < PseudoPrimeGenerator
     def initialize
       @index = -1
       super
@@ -344,7 +345,7 @@ class Prime
   # This is a pseudo-prime generator, suitable on
   # checking primality of an integer by brute force
   # method.
-  class Generator23<PseudoPrimeGenerator
+  class Generator23 < PseudoPrimeGenerator
     def initialize
       @prime = 1
       @step = nil
@@ -418,7 +419,7 @@ class Prime
     end
   end
 
-  # Internal use. An implementation of eratosthenes' sieve
+  # Internal use. An implementation of Eratosthenes' sieve
   class EratosthenesSieve
     include Singleton
 
@@ -444,7 +445,7 @@ class Prime
 
       segment_min = @max_checked
       segment_max = [segment_min + max_segment_size, max_cached_prime * 2].min
-      root = Integer(Math.sqrt(segment_max).floor)
+      root = Integer.sqrt(segment_max)
 
       segment = ((segment_min + 1) .. segment_max).step(2).to_a
 
